--[[============================================================================
main.lua
============================================================================]]--

-- Placeholder for the dialog
local dialog = nil

-- Placeholder to expose the ViewBuilder outside the show_dialog() function
local vb = nil

--Note-matrix attributes
note_matrix = {
   [1]='C', [2]='C#', [3]='D', [4]='D#', [5]='E', [6]='F',
   [7]='F#', [8]='G', [9]='G#', [10]='A', [11]='A#', [12]='B'
}

-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
  
end

-- Read from the manifest.xml file.
class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
  function RenoiseScriptingTool:__init()    
    renoise.Document.DocumentNode.__init(self) 
    self:add_property("Name", "Untitled Tool")
    self:add_property("Id", "Unknown Id")
  end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_id = manifest:property("Id").value


--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------

function key_handler(dialog, key)
  -- close on escape...

  if (key.modifiers == "" and key.name == "esc") then
    dialog:close()

  -- Let's send the text-line contents if present
  elseif (key.name == "return") then
--    no_loop = 1
    vb.views.note_views.text = ""
  
  -- update key_text to show what we got
  elseif (key.name == "back") then
--    no_loop = 1
    vb.views.note_views.text = string.sub(vb.views.note_views.text,1,
    string.len(vb.views.note_views.text)-1)

  elseif (key.name == "del") then
--    no_loop = 1
    vb.views.note_views.text = ""

  elseif key.character ~= nil then
--    no_loop = 1
    vb.views.note_views.text = vb.views.note_views.text .. 
      key.character
  end
end

function fix_full_note(tempvar)
   local valid_note_found = false
   tempvar = string.sub(tempvar,1,3)

   --Are we tricked with invalid split-chars?
   if string.sub(tempvar,2,2) ~= "#" and string.sub(tempvar,2,2) ~= "-" then
      tempvar = string.sub(tempvar,1,1) .. "-" .. string.sub(tempvar,3,3)
--      custom_error_flag = true
   end

   --Are we tricked with invalid octave figures?
   if tonumber(string.sub(tempvar,3,3)) == nil then
      tempvar = string.sub(tempvar,1,2) .. "4"
--      custom_error_flag = true
   end

   --Are we tricked with invalid note figures?
   for i = 1,12 do

      if string.upper(string.sub(tempvar,1,1)) == string.sub(note_matrix[i],1,1) then
         valid_note_found = true
         break
      end

   end

   if valid_note_found == false then
      tempvar = "C" .. string.sub(tempvar,2,3)
--      custom_error_flag = true
   end   

   --take care only the first three characters of the tempvar are returned
   tempvar = string.sub(tempvar,1,3)
   return tempvar
end

--------------------------------------------------------------------------------
-- GUI
--------------------------------------------------------------------------------

local function show_dialog()

  -- This block makes sure a non-modal dialog is shown once.
  -- If the dialog is already opened, it will be focused.
  if dialog and dialog.visible then
    dialog:show()
    return
  end

  local note_column = renoise.song().selected_note_column_index
  local track = renoise.song().selected_track_index
  local line = renoise.song().selected_line_index
  local pattern = renoise.song().selected_pattern_index
  local selected_line = renoise.song().patterns[pattern].tracks[track].lines[line]
  local note = selected_line.note_columns[note_column].note_string

  -- The ViewBuilder is the basis
  vb = renoise.ViewBuilder()
  
  -- The content of the dialog, built with the ViewBuilder.
  local content = vb:column {
    margin = 10,
    vb:textfield {
      width = 40,

      id="note_views",
      text = note,
      notifier = function(text)
        if string.len(text) <4 then
          
          if string.len(text) == 3 then
            text = fix_full_note(text)
            selected_line.note_columns[note_column].note_string = text
            if dialog ~= nil then
              if dialog.visible then
                dialog:close()
                vb = nil
              end
            end
          end 
        end
      end
    }
  } 
  
  -- A custom dialog is non-modal and displays a user designed
  -- layout built with the ViewBuilder.   
  dialog = renoise.app():show_custom_dialog(tool_name, content, key_handler)  

end


--------------------------------------------------------------------------------
-- Menu entries
--------------------------------------------------------------------------------

renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:"..tool_name.."...",
  invoke = show_dialog  
}


--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------

renoise.tool():add_keybinding {
  name = "Global:Tools:" .. tool_name.."...",
  invoke = show_dialog
}


--------------------------------------------------------------------------------
-- MIDI Mapping
--------------------------------------------------------------------------------

--[[
renoise.tool():add_midi_mapping {
  name = tool_id..":Show Dialog...",
  invoke = show_dialog
}
--]]
